// SN74LV8153 7seg LED x2 display test
//
// dsPIC30F2012 UTX (17pin) ---> SN74LV8153 D (5pin)
// dsPIC30F2012 RF6 (16pin) ---> SN74LV8153 RESET# (7pin)

#include <p30F2012.h>

// '0'-'9', 'A'-'F' 7seg LED data
unsigned char hex[16]={0x77, 0x41, 0x3b, 0x6b, 0x4d, 0x6e, 0x7e, 0x43,
                       0x7f, 0x4f, 0x5f, 0x7c, 0x36, 0x79, 0x3e, 0x1e};

//----------------------------------------

void wait(unsigned x) {
  unsigned i, j;

  for (i=0; i<x; i=i+1) {
    for (j=0; j<60000; j=j+1) asm("nop");
  }
}

//----------------------------------------

void led(unsigned char digit, unsigned char x) {
  unsigned char adrs;

  if (digit==0) {               // if 1st digit
    adrs=1;
  } else {                      // 2nd digit
    adrs=3;
  }
  while(U1STAbits.UTXBF!=0);    // wait until transmit finish
  U1TXREG=adrs+(x<<4);          // send 1st frame data 
  while(U1STAbits.UTXBF!=0);    // wait until transmit finish
  U1TXREG=adrs+((x>>4)<<4);     // send 2nd frame data
}

//----------------------------------------

void led_hex(unsigned char digit, unsigned char x,
             unsigned char dp) {
  if (dp==0) {
    led(digit, hex[x]);
  } else {
    led(digit, hex[x]+0x80);
  }
}

//----------------------------------------

void led_byte(unsigned char x) {    // x=0-255 (0x00-0xFF)
  led_hex(0, x&0x0f, 0);
  led_hex(1, x>>4, 0);
}

//----------------------------------------

int main(void) {
  unsigned i;

  _TRISF6=0;       // RF6 is output (reset#)

  U1BRG =95;       // 19200bps (use arbitrary baud rate!)
  U1MODE=0x8000;   // N81 (Non Parity, Data 8bit, Stop Bit 1bit)
  U1STA =0x0400;   // use RF3 as UTX (UART output)

  _RF6=0;          // SN74LV8153 reset
  wait(10);        // wait...
  _RF6=1;          // SN74LV8153 release reset

  i=0;
  while(1) {       // forever loop
    i=(i+1)%256;   // count up
    led_byte(i);   // display '00'-'FF'
    wait(20);      // wait
  }  

}






