// MAX7219 7seg LED drive sample program
//
// RB6 : DATA
// RB7 : LOAD
// RB8 : CLOCK

#include <p30F2012.h>

//----------------------------------------------------------

void wait(unsigned x) {      // wait function (delay)
  unsigned i;

  for (i=0; i<x; i=i+1) asm("nop");
}

//----------------------------------------------------------

void init_led_port(void) {
  _RB7=1;     // RB6 LOAD=1
  _RB8=0;     // RB8 CLOCK=0
  _TRISB6=0;  // RB6 DATA output
  _TRISB7=0;  // RB7 LOAD output
  _TRISB8=0;  // RB8 CLOCK output
}

//----------------------------------------------------------

void send7219(unsigned char adrs, unsigned char data) {
  unsigned i;

  _RB7=0;                    // RB7 LOAD=0
  // write address
  for (i=0; i<8; i=i+1) {
    wait(10);
    _RB8=0;                  // RB8 CLOCK=0
    wait(10);
    if (0x80<=adrs) {        // send address bit
      _RB6=1;
    } else {
      _RB6=0;
    }
    adrs=adrs<<1;            // shift address bits
    wait(10);
    _RB8=1;                  // RB8 CLOCK=1
  }
  // write data
  for (i=0; i<8; i=i+1) {
    wait(10);
    _RB8=0;                  // RB8 CLOCK=0
    wait(10);
    if (0x80<=data) {        // send data bit
      _RB6=1;
    } else {
      _RB6=0;
    }
    data=data<<1;            // shift data bits
    wait(10);
    _RB8=1;                  // RB8 CLOCK=1
  }    
  wait(10);
  _RB7=1;                    // RB7 LOAD=1
  wait(10);
  _RB8=0;                    // RB8 CLOCK=1
  wait(10);
}

//----------------------------------------------------------

unsigned time;

void main(void) {
  unsigned i;

  init_led_port();
  
  send7219(0x0f, 0x00);      // display test reg - normal operation
  for (i=1; i<=8; i=i+1) {
    send7219(i, 0);          // clear data reg
  }
  send7219(0x09, 0xff);      // decode moode reg - all decode
  send7219(0x0a, 0x0f);      // intensity reg - maximum brightness
  send7219(0x0b, 0x06);      // scan limit reg - 7digit
  send7219(0x0c, 0x01);      // shutdown reg - normal operation

  time=0;
  while(1) {
    for (i=0; i<20; i=i+1) wait(60000);
    time=(time+1)%16;
    for (i=0; i<7; i=i+1) {
      send7219(i+1, (time+i)%16);
    }
  }

}
